from rest_framework import status as st
from rest_framework.response import Response
import json


class JsonRequest:
    MODEL = None
    DATA = None
    OPTIONS = None
    MAX_LENGTH = None
    UNDEFINED_FIELD = False
    MODEL_TYPE = [int, str, float, bool, list, dict, tuple, set, "ANY"]

    def __init__(self, data=None):
        self.load_data(data)

    def load_data(self, data):
        self.body = data if data else json.loads(data.decode('utf-8'))

    def check_configurations(self):
        if self.MODEL is not None:
            for mdl in self.MODEL:
                if self.MODEL[mdl] not in self.MODEL_TYPE:
                    raise Exception(f"Model type must be in {self.MODEL_TYPE}")
            if self.OPTIONS is not None:
                for opt in self.OPTIONS:
                    if self.MODEL.get(opt) is None:
                        raise Exception("Option key must be in model")
            if self.MAX_LENGTH is not None:
                for opt in self.MAX_LENGTH:
                    if self.MODEL.get(opt) is None:
                        raise Exception("Max length key must be in model")
                    if not isinstance(self.MODEL[opt], str):
                        raise Exception("Max length key must be str")
                    if not isinstance(self.MAX_LENGTH[opt], int):
                        raise Exception("Max length value must be int")
            return True
        raise Exception("Model is not defined")

    def check_field(self):
        if self.check_configurations():
            if not self.UNDEFINED_FIELD:
                if not len(self.MODEL) == len(self.body):
                    return False
            for bdy in self.body:
                if bdy not in self.MODEL:
                    return False
                if not self.MODEL[bdy] == "ANY":
                    if not isinstance(self.body[bdy], self.MODEL[bdy]):
                        return False
                if self.OPTIONS is not None:
                    if bdy in self.OPTIONS:
                        if self.body[bdy] not in self.OPTIONS[bdy]:
                            return False
                if self.MAX_LENGTH is not None:
                    if bdy in self.MAX_LENGTH:
                        if len(self.body[bdy]) > self.MAX_LENGTH[bdy]:
                            return False
            self.DATA = self.body
            self.body = None
            return True

    def is_valid(self):
        if self.body:
            return self.check_field()
        raise Exception("Body is empty")

    def response(self, data=None, status=st.HTTP_200_OK):
        return Response(data=data, status=status)

    def lazy_response(self, function, parameters: set, data=None, status=st.HTTP_200_OK):
        return LazyResponse(data=data, then_callback=function, status=status, parameters=parameters)

    def http_100_continue(self, data=None):
        return Response(data=data, status=st.HTTP_100_CONTINUE)

    def http_101_switching_protocols(self, data=None):
        return Response(data=data, status=st.HTTP_101_SWITCHING_PROTOCOLS)

    def http_102_processing(self, data=None):
        return Response(data=data, status=st.HTTP_102_PROCESSING)

    def http_103_early_hints(self, data=None):
        return Response(data=data, status=st.HTTP_103_EARLY_HINTS)

    def http_200_ok(self, data=None):
        return Response(data=data, status=st.HTTP_200_OK)

    def http_201_created(self, data=None):
        return Response(data=data, status=st.HTTP_201_CREATED)

    def http_202_accepted(self, data=None):
        return Response(data=data, status=st.HTTP_202_ACCEPTED)

    def http_203_non_authoritative_information(self, data=None):
        return Response(data=data, status=st.HTTP_203_NON_AUTHORITATIVE_INFORMATION)

    def http_204_no_content(self, data=None):
        return Response(data=data, status=st.HTTP_204_NO_CONTENT)

    def http_205_reset_content(self, data=None):
        return Response(data=data, status=st.HTTP_205_RESET_CONTENT)

    def http_206_partial_content(self, data=None):
        return Response(data=data, status=st.HTTP_206_PARTIAL_CONTENT)

    def http_207_multi_status(self, data=None):
        return Response(data=data, status=st.HTTP_207_MULTI_STATUS)

    def http_208_already_reported(self, data=None):
        return Response(data=data, status=st.HTTP_208_ALREADY_REPORTED)

    def http_226_im_used(self, data=None):
        return Response(data=data, status=st.HTTP_226_IM_USED)

    def http_300_multiple_choices(self, data=None):
        return Response(data=data, status=st.HTTP_300_MULTIPLE_CHOICES)

    def http_301_moved_permanently(self, data=None):
        return Response(data=data, status=st.HTTP_301_MOVED_PERMANENTLY)

    def http_302_found(self, data=None):
        return Response(data=data, status=st.HTTP_302_FOUND)

    def http_303_see_other(self, data=None):
        return Response(data=data, status=st.HTTP_303_SEE_OTHER)

    def http_304_not_modified(self, data=None):
        return Response(data=data, status=st.HTTP_304_NOT_MODIFIED)

    def http_305_use_proxy(self, data=None):
        return Response(data=data, status=st.HTTP_305_USE_PROXY)

    def http_306_switch_proxy(self, data=None):
        return Response(data=data, status=st.HTTP_306_RESERVED)

    def http_307_temporary_redirect(self, data=None):
        return Response(data=data, status=st.HTTP_307_TEMPORARY_REDIRECT)

    def http_308_permanent_redirect(self, data=None):
        return Response(data=data, status=st.HTTP_308_PERMANENT_REDIRECT)

    def http_400_bad_request(self, data=None):
        return Response(data=data, status=st.HTTP_400_BAD_REQUEST)

    def http_401_unauthorized(self, data=None):
        return Response(data=data, status=st.HTTP_401_UNAUTHORIZED)

    def http_402_payment_required(self, data=None):
        return Response(data=data, status=st.HTTP_402_PAYMENT_REQUIRED)

    def http_403_forbidden(self, data=None):
        return Response(data=data, status=st.HTTP_403_FORBIDDEN)

    def http_404_not_found(self, data=None):
        return Response(data=data, status=st.HTTP_404_NOT_FOUND)

    def http_405_method_not_allowed(self, data=None):
        return Response(data=data, status=st.HTTP_405_METHOD_NOT_ALLOWED)

    def http_406_not_acceptable(self, data=None):
        return Response(data=data, status=st.HTTP_406_NOT_ACCEPTABLE)

    def http_407_proxy_authentication_required(self, data=None):
        return Response(data=data, status=st.HTTP_407_PROXY_AUTHENTICATION_REQUIRED)

    def http_408_request_timeout(self, data=None):
        return Response(data=data, status=st.HTTP_408_REQUEST_TIMEOUT)

    def http_409_conflict(self, data=None):
        return Response(data=data, status=st.HTTP_409_CONFLICT)

    def http_410_gone(self, data=None):
        return Response(data=data, status=st.HTTP_410_GONE)

    def http_411_length_required(self, data=None):
        return Response(data=data, status=st.HTTP_411_LENGTH_REQUIRED)

    def http_412_precondition_failed(self, data=None):
        return Response(data=data, status=st.HTTP_412_PRECONDITION_FAILED)

    def http_413_payload_too_large(self, data=None):
        return Response(data=data, status=st.HTTP_413_REQUEST_ENTITY_TOO_LARGE)

    def http_414_uri_too_long(self, data=None):
        return Response(data=data, status=st.HTTP_414_REQUEST_URI_TOO_LONG)

    def http_415_unsupported_media_type(self, data=None):
        return Response(data=data, status=st.HTTP_415_UNSUPPORTED_MEDIA_TYPE)

    def http_416_requested_range_not_satisfiable(self, data=None):
        return Response(data=data, status=st.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE)

    def http_417_expectation_failed(self, data=None):
        return Response(data=data, status=st.HTTP_417_EXPECTATION_FAILED)

    def http_418_im_a_teapot(self, data=None):
        return Response(data=data, status=st.HTTP_418_IM_A_TEAPOT)

    def http_421_misdirected_request(self, data=None):
        return Response(data=data, status=st.HTTP_421_MISDIRECTED_REQUEST)

    def http_422_unprocessable_entity(self, data=None):
        return Response(data=data, status=st.HTTP_422_UNPROCESSABLE_ENTITY)

    def http_423_locked(self, data=None):
        return Response(data=data, status=st.HTTP_423_LOCKED)

    def http_424_failed_dependency(self, data=None):
        return Response(data=data, status=st.HTTP_424_FAILED_DEPENDENCY)

    def http_426_upgrade_required(self, data=None):
        return Response(data=data, status=st.HTTP_426_UPGRADE_REQUIRED)

    def http_428_precondition_required(self, data=None):
        return Response(data=data, status=st.HTTP_428_PRECONDITION_REQUIRED)

    def http_429_too_many_requests(self, data=None):
        return Response(data=data, status=st.HTTP_429_TOO_MANY_REQUESTS)

    def http_431_request_header_fields_too_large(self, data=None):
        return Response(data=data, status=st.HTTP_431_REQUEST_HEADER_FIELDS_TOO_LARGE)

    def http_451_unavailable_for_legal_reasons(self, data=None):
        return Response(data=data, status=st.HTTP_451_UNAVAILABLE_FOR_LEGAL_REASONS)

    def http_500_internal_server_error(self, data=None):
        return Response(data=data, status=st.HTTP_500_INTERNAL_SERVER_ERROR)

    def http_501_not_implemented(self, data=None):
        return Response(data=data, status=st.HTTP_501_NOT_IMPLEMENTED)

    def http_502_bad_gateway(self, data=None):
        return Response(data=data, status=st.HTTP_502_BAD_GATEWAY)

    def http_503_service_unavailable(self, data=None):
        return Response(data=data, status=st.HTTP_503_SERVICE_UNAVAILABLE)

    def http_504_gateway_timeout(self, data=None):
        return Response(data=data, status=st.HTTP_504_GATEWAY_TIMEOUT)

    def http_505_http_version_not_supported(self, data=None):
        return Response(data=data, status=st.HTTP_505_HTTP_VERSION_NOT_SUPPORTED)

    def http_506_variant_also_negotiates(self, data=None):
        return Response(data=data, status=st.HTTP_506_VARIANT_ALSO_NEGOTIATES)

    def http_507_insufficient_storage(self, data=None):
        return Response(data=data, status=st.HTTP_507_INSUFFICIENT_STORAGE)

    def http_508_loop_detected(self, data=None):
        return Response(data=data, status=st.HTTP_508_LOOP_DETECTED)

    def http_510_not_extended(self, data=None):
        return Response(data=data, status=st.HTTP_510_NOT_EXTENDED)

    def http_511_network_authentication_required(self, data=None):
        return Response(data=data, status=st.HTTP_511_NETWORK_AUTHENTICATION_REQUIRED)


class LazyResponse(Response):
    def __init__(self, data, then_callback, parameters, **kwargs):
        super().__init__(data, **kwargs)
        self.then_callback = then_callback
        self.parameters = parameters

    def close(self):
        super().close()
        self.then_callback(*self.parameters)
